#pragma once

#ifndef IMM_EVENT_MANAGER_H_
#define IMM_EVENT_MANAGER_H_

#include "immersitech.h"

/// @brief All possible events that an Immersitech Event manager can support
///
typedef enum {
	IMM_EVENT_CREATE_ROOM,					///< An event triggered when a room is created
	IMM_EVENT_DESTROY_ROOM,					///< An event triggered when a room is destroyed
	IMM_EVENT_ADD_PARTICIPANT,				///< An event triggered when a participant is added to a room
	IMM_EVENT_REMOVE_PARTICIPANT,			///< An event triggered when a participant is removed from a room
	IMM_EVENT_SET_PARTICIPANT_SEAT,			///< An event triggered when a participant changes seats
	IMM_EVENT_SET_PARTICIPANT_POSITION,		///< An event triggered when a participant changes position
	IMM_EVENT_SET_PARTICIPANT_STATE,		///< An event triggered when a participant changes the value of an audio control
	IMM_EVENT_SET_ALL_PARTICIPANTS_STATE,	///< An event triggered when all participants change the value of an audio control
	IMM_EVENT_SET_ROOM_LAYOUT,				///< An event triggered when a room has its layout changed
	IMM_EVENT_SET_NAME						///< An event triggered when a participant's name changes
} imm_event_type;

/// @brief Default immersitech event manager.
///
/// A custom event manager can be implemented by passing in a pointer to an instance of the data structure imm_event_manager. 
/// The developer must link each member of the data structure to an appropriate implementation of a callback function for the
/// particular type of event.

/// @brief Function pointer type for callback to propagate an IMM_EVENT_CREATE_ROOM event
///
/// @param[in] room_id The ID of the room that was created
typedef void (*create_room_event_callback)(int room_id);

/// @brief Function pointer type for callback to propagate an IMM_EVENT_DESTROY_ROOM event
///
/// @param[in] room_id The ID of the room that was destroyed
typedef void (*destroy_room_event_callback)(int room_id);

/// @brief Function pointer type for callback to propagate an IMM_EVENT_ADD_PARTICIPANT event
///
/// @param[in] room_id The ID of the room that the participant was added to
/// @param[in] participant_id The ID of the participant that was added
/// @param[in] participant_name The name of the participant that was added
/// @param[in] config The configuration for the participant such as input sampling rate and input number of frames
typedef void (*add_participant_event_callback)(int room_id, int participant_id, const char* participant_name, imm_participant_configuration config);

/// @brief Function pointer type for callback to propagate an IMM_EVENT_REMOVE_PARTICIPANT event
///
/// @param[in] room_id The ID of the room that the participant was removed from
/// @param[in] participant_id The ID of the participant that was removed
typedef void (*remove_participant_event_callback)(int room_id, int participant_id);

/// @brief Function pointer type for callback to propagate an IMM_EVENT_SET_PARTICIPANT_SEAT event
///
/// @param[in] room_id The ID of the room that the participant was moved in
/// @param[in] participant_id The ID of the participant that was moved
/// @param[in] seat The seat that the participant was moved to
typedef void (*set_participant_seat_event_callback)(int room_id, int participant_id, imm_seat seat);

/// @brief Function pointer type for callback to propagate an IMM_EVENT_SET_PARTICIPANT_POSITION event
///
/// @param[in] room_id The ID of the room that the participant was moved in
/// @param[in] participant_id The ID of the participant that was moved
/// @param[in] position The new position of the participant
/// @param[in] heading The new heading of the participant
typedef void (*set_participant_position_event_callback)(int room_id, int participant_id, imm_position position, imm_heading heading);

/// @brief Function pointer type for callback to propagate an IMM_EVENT_SET_PARTICIPANT_STATE event
///
/// @param[in] room_id The ID of the room that the participant is in
/// @param[in] participant_id The ID of the participant whose state was changed
/// @param[in] control_to_edit The audio control the was changed
/// @param[in] value The value the audio control was set to
typedef void (*set_participant_state_event_callback)(int room_id, int participant_id, imm_audio_control control_to_edit, int value);

/// @brief Function pointer type for callback to propagate an IMM_EVENT_SET_ALL_PARTICIPANTS_STATE event
///
/// @param[in] room_id The ID of the room that the participants are in
/// @param[in] control_to_edit The audio control that was changed
/// @param[in] value The value the audio control was set to
typedef void (*set_all_participants_state_event_callback)(int room_id, imm_audio_control control_to_edit, int value);

/// @brief Function pointer type for callback to propagate an IMM_EVENT_SET_ROOM_LAYOUT event
///
/// @param[in] room_id The ID of the room that changed its layout
/// @param[in] room_layout_id The ID of the room layout that was applied
typedef void (*set_room_layout_event_callback)(int room_id, int room_layout_id);

/// @brief Function pointer type for callback to propagate an IMM_EVENT_SET_NAME event
///
/// @param[in] room_id The ID of the room
/// @param[in] participant_id The ID of the participant whose name changed
/// @param[in] name The new name for the participant
typedef void (*set_name_event_callback)(int room_id, int participant_id, const char* name);

typedef struct imm_event_manager_t
{
	create_room_event_callback cb_room_created;
	destroy_room_event_callback cb_room_destroyed;
	add_participant_event_callback cb_participant_added;
	remove_participant_event_callback cb_participant_removed;
	set_participant_seat_event_callback cb_participant_seat_changed;
	set_participant_position_event_callback cb_participant_position_changed;
	set_participant_state_event_callback cb_participant_state_changed;
	set_all_participants_state_event_callback cb_participants_state_changed;
	set_room_layout_event_callback cb_room_layout_set;
	set_name_event_callback cb_participant_name_changed;
} imm_event_manager;

#endif //IMM_EVENT_MANAGER_H_